@@ -1,11 +1,12 @@
+import operator
 import sys
+
 import pytest
 
-import operator
 import numpy as np
-
-from numpy.testing import assert_array_equal, assert_raises, IS_PYPY
-
+from numpy._core._exceptions import _UFuncNoLoopError
+from numpy.testing import IS_PYPY, assert_array_equal, assert_raises
+from numpy.testing._private.utils import requires_memory
 
 COMPARISONS = [
     (operator.eq, np.equal, "=="),
@@ -109,6 +110,88 @@ def test_float_to_string_cast(str_dt, float_dt):
     assert_array_equal(res, np.array(expected, dtype=str_dt))
 
 
+@pytest.mark.parametrize("str_dt", "US")
+@pytest.mark.parametrize("size", [-1, np.iinfo(np.intc).max])
+def test_string_size_dtype_errors(str_dt, size):
+    if size > 0:
+        size = size // np.dtype(f"{str_dt}1").itemsize + 1
+
+    with pytest.raises(ValueError):
+        np.dtype((str_dt, size))
+    with pytest.raises(TypeError):
+        np.dtype(f"{str_dt}{size}")
+
+
+@pytest.mark.parametrize("str_dt", "US")
+def test_string_size_dtype_large_repr(str_dt):
+    size = np.iinfo(np.intc).max // np.dtype(f"{str_dt}1").itemsize
+    size_str = str(size)
+
+    dtype = np.dtype((str_dt, size))
+    assert size_str in dtype.str
+    assert size_str in str(dtype)
+    assert size_str in repr(dtype)
+
+
+@pytest.mark.slow
+@requires_memory(2 * np.iinfo(np.intc).max)
+@pytest.mark.parametrize("str_dt", "US")
+def test_large_string_coercion_error(str_dt):
+    very_large = np.iinfo(np.intc).max // np.dtype(f"{str_dt}1").itemsize
+    try:
+        large_string = "A" * (very_large + 1)
+    except Exception:
+        # We may not be able to create this Python string on 32bit.
+        pytest.skip("python failed to create huge string")
+
+    class MyStr:
+        def __str__(self):
+            return large_string
+
+    try:
+        # TypeError from NumPy, or OverflowError from 32bit Python.
+        with pytest.raises((TypeError, OverflowError)):
+            np.array([large_string], dtype=str_dt)
+
+        # Same as above, but input has to be converted to a string.
+        with pytest.raises((TypeError, OverflowError)):
+            np.array([MyStr()], dtype=str_dt)
+    except MemoryError:
+        # Catch memory errors, because `requires_memory` would do so.
+        raise AssertionError("Ops should raise before any large allocation.")
+
+@pytest.mark.slow
+@requires_memory(2 * np.iinfo(np.intc).max)
+@pytest.mark.parametrize("str_dt", "US")
+def test_large_string_addition_error(str_dt):
+    very_large = np.iinfo(np.intc).max // np.dtype(f"{str_dt}1").itemsize
+
+    a = np.array(["A" * very_large], dtype=str_dt)
+    b = np.array("B", dtype=str_dt)
+    try:
+        with pytest.raises(TypeError):
+            np.add(a, b)
+        with pytest.raises(TypeError):
+            np.add(a, a)
+    except MemoryError:
+        # Catch memory errors, because `requires_memory` would do so.
+        raise AssertionError("Ops should raise before any large allocation.")
+
+
+def test_large_string_cast():
+    very_large = np.iinfo(np.intc).max // 4
+    # Could be nice to test very large path, but it makes too many huge
+    # allocations right now (need non-legacy cast loops for this).
+    # a = np.array([], dtype=np.dtype(("S", very_large)))
+    # assert a.astype("U").dtype.itemsize == very_large * 4
+
+    a = np.array([], dtype=np.dtype(("S", very_large + 1)))
+    # It is not perfect but OK if this raises a MemoryError during setup
+    # (this happens due clunky code and/or buffer setup.)
+    with pytest.raises((TypeError, MemoryError)):
+        a.astype("U")
+
+
 @pytest.mark.parametrize("dt", ["S", "U", "T"])
 class TestMethods:
 
@@ -142,9 +225,20 @@ class TestMethods:
         with pytest.raises(TypeError, match="unsupported type"):
             np.strings.multiply(np.array("abc", dtype=dt), 3.14)
 
-        with pytest.raises(MemoryError):
+        with pytest.raises(OverflowError):
             np.strings.multiply(np.array("abc", dtype=dt), sys.maxsize)
 
+    def test_inplace_multiply(self, dt):
+        arr = np.array(['foo ', 'bar'], dtype=dt)
+        arr *= 2
+        if dt != "T":
+            assert_array_equal(arr, np.array(['foo ', 'barb'], dtype=dt))
+        else:
+            assert_array_equal(arr, ['foo foo ', 'barbar'])
+
+        with pytest.raises(OverflowError):
+            arr *= sys.maxsize
+
     @pytest.mark.parametrize("i_dt", [np.int8, np.int16, np.int32,
                                       np.int64, np.int_])
     def test_multiply_integer_dtypes(self, i_dt, dt):
@@ -281,24 +375,26 @@ class TestMethods:
         ("", "xx", 0, None, -1),
         ("", "xx", 1, 1, -1),
         ("", "xx", MAX, 0, -1),
-        pytest.param(99*"a" + "b", "b", 0, None, 99,
+        pytest.param(99 * "a" + "b", "b", 0, None, 99,
                      id="99*a+b-b-0-None-99"),
-        pytest.param(98*"a" + "ba", "ba", 0, None, 98,
+        pytest.param(98 * "a" + "ba", "ba", 0, None, 98,
                      id="98*a+ba-ba-0-None-98"),
-        pytest.param(100*"a", "b", 0, None, -1,
+        pytest.param(100 * "a", "b", 0, None, -1,
                      id="100*a-b-0-None--1"),
-        pytest.param(30000*"a" + 100*"b", 100*"b", 0, None, 30000,
+        pytest.param(30000 * "a" + 100 * "b", 100 * "b", 0, None, 30000,
                      id="30000*a+100*b-100*b-0-None-30000"),
-        pytest.param(30000*"a", 100*"b", 0, None, -1,
+        pytest.param(30000 * "a", 100 * "b", 0, None, -1,
                      id="30000*a-100*b-0-None--1"),
-        pytest.param(15000*"a" + 15000*"b", 15000*"b", 0, None, 15000,
+        pytest.param(15000 * "a" + 15000 * "b", 15000 * "b", 0, None, 15000,
                      id="15000*a+15000*b-15000*b-0-None-15000"),
-        pytest.param(15000*"a" + 15000*"b", 15000*"c", 0, None, -1,
+        pytest.param(15000 * "a" + 15000 * "b", 15000 * "c", 0, None, -1,
                      id="15000*a+15000*b-15000*c-0-None--1"),
         (["abcdefghiabc", "rrarrrrrrrrra"], ["def", "arr"], [0, 3],
          None, [3, -1]),
         ("Ae¢☃€ 😊" * 2, "😊", 0, None, 6),
         ("Ae¢☃€ 😊" * 2, "😊", 7, None, 13),
+        pytest.param("A" * (2 ** 17), r"[\w]+\Z", 0, None, -1,
+                     id=r"A*2**17-[\w]+\Z-0-None--1"),
     ])
     def test_find(self, a, sub, start, end, out, dt):
         if "😊" in a and dt == "S":
@@ -347,17 +443,17 @@ class TestMethods:
         ("aaa", "", -1, None, 2),
         ("aaa", "", -10, None, 4),
         ("aaa", "aaaa", 0, None, 0),
-        pytest.param(98*"a" + "ba", "ba", 0, None, 1,
+        pytest.param(98 * "a" + "ba", "ba", 0, None, 1,
                      id="98*a+ba-ba-0-None-1"),
-        pytest.param(30000*"a" + 100*"b", 100*"b", 0, None, 1,
+        pytest.param(30000 * "a" + 100 * "b", 100 * "b", 0, None, 1,
                      id="30000*a+100*b-100*b-0-None-1"),
-        pytest.param(30000*"a", 100*"b", 0, None, 0,
+        pytest.param(30000 * "a", 100 * "b", 0, None, 0,
                      id="30000*a-100*b-0-None-0"),
-        pytest.param(30000*"a" + 100*"ab", "ab", 0, None, 100,
+        pytest.param(30000 * "a" + 100 * "ab", "ab", 0, None, 100,
                      id="30000*a+100*ab-ab-0-None-100"),
-        pytest.param(15000*"a" + 15000*"b", 15000*"b", 0, None, 1,
+        pytest.param(15000 * "a" + 15000 * "b", 15000 * "b", 0, None, 1,
                      id="15000*a+15000*b-15000*b-0-None-1"),
-        pytest.param(15000*"a" + 15000*"b", 15000*"c", 0, None, 0,
+        pytest.param(15000 * "a" + 15000 * "b", 15000 * "c", 0, None, 0,
                      id="15000*a+15000*b-15000*c-0-None-0"),
         ("", "", 0, None, 1),
         ("", "", 1, 1, 0),
@@ -566,7 +662,7 @@ class TestMethods:
         ("ABCADAA", "A", "", -1, "BCD"),
         ("BCD", "A", "", -1, "BCD"),
         ("*************", "A", "", -1, "*************"),
-        ("^"+"A"*1000+"^", "A", "", 999, "^A^"),
+        ("^" + "A" * 1000 + "^", "A", "", 999, "^A^"),
         ("the", "the", "", -1, ""),
         ("theater", "the", "", -1, "ater"),
         ("thethe", "the", "", -1, ""),
@@ -726,6 +822,208 @@ class TestMethods:
             np.strings.expandtabs(np.array("\ta\n\tb", dtype=dt), sys.maxsize)
             np.strings.expandtabs(np.array("\ta\n\tb", dtype=dt), 2**61)
 
+    def test_expandtabs_length_not_cause_segfault(self, dt):
+        # see gh-28829
+        with pytest.raises(
+            _UFuncNoLoopError,
+            match="did not contain a loop with signature matching types",
+        ):
+            np._core.strings._expandtabs_length.reduce(np.zeros(200))
+
+        with pytest.raises(
+            _UFuncNoLoopError,
+            match="did not contain a loop with signature matching types",
+        ):
+            np.strings.expandtabs(np.zeros(200))
+
+    FILL_ERROR = "The fill character must be exactly one character long"
+
+    def test_center_raises_multiple_character_fill(self, dt):
+        buf = np.array("abc", dtype=dt)
+        fill = np.array("**", dtype=dt)
+        with pytest.raises(TypeError, match=self.FILL_ERROR):
+            np.strings.center(buf, 10, fill)
+
+    def test_ljust_raises_multiple_character_fill(self, dt):
+        buf = np.array("abc", dtype=dt)
+        fill = np.array("**", dtype=dt)
+        with pytest.raises(TypeError, match=self.FILL_ERROR):
+            np.strings.ljust(buf, 10, fill)
+
+    def test_rjust_raises_multiple_character_fill(self, dt):
+        buf = np.array("abc", dtype=dt)
+        fill = np.array("**", dtype=dt)
+        with pytest.raises(TypeError, match=self.FILL_ERROR):
+            np.strings.rjust(buf, 10, fill)
+
+    @pytest.mark.parametrize("buf,width,fillchar,res", [
+        ('abc', 10, ' ', '   abc    '),
+        ('abc', 6, ' ', ' abc  '),
+        ('abc', 3, ' ', 'abc'),
+        ('abc', 2, ' ', 'abc'),
+        ('abc', -2, ' ', 'abc'),
+        ('abc', 10, '*', '***abc****'),
+    ])
+    def test_center(self, buf, width, fillchar, res, dt):
+        buf = np.array(buf, dtype=dt)
+        fillchar = np.array(fillchar, dtype=dt)
+        res = np.array(res, dtype=dt)
+        assert_array_equal(np.strings.center(buf, width, fillchar), res)
+
+    @pytest.mark.parametrize("buf,width,fillchar,res", [
+        ('abc', 10, ' ', 'abc       '),
+        ('abc', 6, ' ', 'abc   '),
+        ('abc', 3, ' ', 'abc'),
+        ('abc', 2, ' ', 'abc'),
+        ('abc', -2, ' ', 'abc'),
+        ('abc', 10, '*', 'abc*******'),
+    ])
+    def test_ljust(self, buf, width, fillchar, res, dt):
+        buf = np.array(buf, dtype=dt)
+        fillchar = np.array(fillchar, dtype=dt)
+        res = np.array(res, dtype=dt)
+        assert_array_equal(np.strings.ljust(buf, width, fillchar), res)
+
+    @pytest.mark.parametrize("buf,width,fillchar,res", [
+        ('abc', 10, ' ', '       abc'),
+        ('abc', 6, ' ', '   abc'),
+        ('abc', 3, ' ', 'abc'),
+        ('abc', 2, ' ', 'abc'),
+        ('abc', -2, ' ', 'abc'),
+        ('abc', 10, '*', '*******abc'),
+    ])
+    def test_rjust(self, buf, width, fillchar, res, dt):
+        buf = np.array(buf, dtype=dt)
+        fillchar = np.array(fillchar, dtype=dt)
+        res = np.array(res, dtype=dt)
+        assert_array_equal(np.strings.rjust(buf, width, fillchar), res)
+
+    @pytest.mark.parametrize("buf,width,res", [
+        ('123', 2, '123'),
+        ('123', 3, '123'),
+        ('0123', 4, '0123'),
+        ('+123', 3, '+123'),
+        ('+123', 4, '+123'),
+        ('+123', 5, '+0123'),
+        ('+0123', 5, '+0123'),
+        ('-123', 3, '-123'),
+        ('-123', 4, '-123'),
+        ('-0123', 5, '-0123'),
+        ('000', 3, '000'),
+        ('34', 1, '34'),
+        ('34', -1, '34'),
+        ('0034', 4, '0034'),
+    ])
+    def test_zfill(self, buf, width, res, dt):
+        buf = np.array(buf, dtype=dt)
+        res = np.array(res, dtype=dt)
+        assert_array_equal(np.strings.zfill(buf, width), res)
+
+    @pytest.mark.parametrize("buf,sep,res1,res2,res3", [
+        ("this is the partition method", "ti", "this is the par",
+            "ti", "tion method"),
+        ("http://www.python.org", "://", "http", "://", "www.python.org"),
+        ("http://www.python.org", "?", "http://www.python.org", "", ""),
+        ("http://www.python.org", "http://", "", "http://", "www.python.org"),
+        ("http://www.python.org", "org", "http://www.python.", "org", ""),
+        ("http://www.python.org", ["://", "?", "http://", "org"],
+            ["http", "http://www.python.org", "", "http://www.python."],
+            ["://", "", "http://", "org"],
+            ["www.python.org", "", "www.python.org", ""]),
+        ("mississippi", "ss", "mi", "ss", "issippi"),
+        ("mississippi", "i", "m", "i", "ssissippi"),
+        ("mississippi", "w", "mississippi", "", ""),
+    ])
+    def test_partition(self, buf, sep, res1, res2, res3, dt):
+        buf = np.array(buf, dtype=dt)
+        sep = np.array(sep, dtype=dt)
+        res1 = np.array(res1, dtype=dt)
+        res2 = np.array(res2, dtype=dt)
+        res3 = np.array(res3, dtype=dt)
+        act1, act2, act3 = np.strings.partition(buf, sep)
+        assert_array_equal(act1, res1)
+        assert_array_equal(act2, res2)
+        assert_array_equal(act3, res3)
+        assert_array_equal(act1 + act2 + act3, buf)
+
+    @pytest.mark.parametrize("buf,sep,res1,res2,res3", [
+        ("this is the partition method", "ti", "this is the parti",
+            "ti", "on method"),
+        ("http://www.python.org", "://", "http", "://", "www.python.org"),
+        ("http://www.python.org", "?", "", "", "http://www.python.org"),
+        ("http://www.python.org", "http://", "", "http://", "www.python.org"),
+        ("http://www.python.org", "org", "http://www.python.", "org", ""),
+        ("http://www.python.org", ["://", "?", "http://", "org"],
+            ["http", "", "", "http://www.python."],
+            ["://", "", "http://", "org"],
+            ["www.python.org", "http://www.python.org", "www.python.org", ""]),
+        ("mississippi", "ss", "missi", "ss", "ippi"),
+        ("mississippi", "i", "mississipp", "i", ""),
+        ("mississippi", "w", "", "", "mississippi"),
+    ])
+    def test_rpartition(self, buf, sep, res1, res2, res3, dt):
+        buf = np.array(buf, dtype=dt)
+        sep = np.array(sep, dtype=dt)
+        res1 = np.array(res1, dtype=dt)
+        res2 = np.array(res2, dtype=dt)
+        res3 = np.array(res3, dtype=dt)
+        act1, act2, act3 = np.strings.rpartition(buf, sep)
+        assert_array_equal(act1, res1)
+        assert_array_equal(act2, res2)
+        assert_array_equal(act3, res3)
+        assert_array_equal(act1 + act2 + act3, buf)
+
+    @pytest.mark.parametrize("args", [
+        (None,),
+        (0,),
+        (1,),
+        (3,),
+        (5,),
+        (6,),  # test index past the end
+        (-1,),
+        (-3,),
+        ([3, 4],),
+        ([2, 4],),
+        ([-3, 5],),
+        ([0, -5],),
+        (1, 4),
+        (-3, 5),
+        (None, -1),
+        (0, [4, 2]),
+        ([1, 2], [-1, -2]),
+        (1, 5, 2),
+        (None, None, -1),
+        ([0, 6], [-1, 0], [2, -1]),
+    ])
+    def test_slice(self, args, dt):
+        buf = np.array(["hello", "world"], dtype=dt)
+        act = np.strings.slice(buf, *args)
+        bcast_args = tuple(np.broadcast_to(arg, buf.shape) for arg in args)
+        res = np.array([s[slice(*arg)]
+                        for s, arg in zip(buf, zip(*bcast_args))],
+                       dtype=dt)
+        assert_array_equal(act, res)
+
+    def test_slice_unsupported(self, dt):
+        with pytest.raises(TypeError, match="did not contain a loop"):
+            np.strings.slice(np.array([1, 2, 3]), 4)
+
+        with pytest.raises(TypeError, match=r"Cannot cast ufunc '_slice' input .* from .* to dtype\('int(64|32)'\)"):
+            np.strings.slice(np.array(['foo', 'bar'], dtype=dt), np.array(['foo', 'bar'], dtype=dt))
+
+    @pytest.mark.parametrize("int_dt", [np.int8, np.int16, np.int32, np.int64,
+                                        np.uint8, np.uint16, np.uint32, np.uint64])
+    def test_slice_int_type_promotion(self, int_dt, dt):
+        buf = np.array(["hello", "world"], dtype=dt)
+
+        assert_array_equal(np.strings.slice(buf, int_dt(4)), np.array(["hell", "worl"], dtype=dt))
+        assert_array_equal(np.strings.slice(buf, np.array([4, 4], dtype=int_dt)), np.array(["hell", "worl"], dtype=dt))
+
+        assert_array_equal(np.strings.slice(buf, int_dt(2), int_dt(4)), np.array(["ll", "rl"], dtype=dt))
+        assert_array_equal(np.strings.slice(buf, np.array([2, 2], dtype=int_dt), np.array([4, 4], dtype=int_dt)), np.array(["ll", "rl"], dtype=dt))
+
+        assert_array_equal(np.strings.slice(buf, int_dt(0), int_dt(4), int_dt(2)), np.array(["hl", "wr"], dtype=dt))
+        assert_array_equal(np.strings.slice(buf, np.array([0, 0], dtype=int_dt), np.array([4, 4], dtype=int_dt), np.array([2, 2], dtype=int_dt)), np.array(["hl", "wr"], dtype=dt))
 
 @pytest.mark.parametrize("dt", ["U", "T"])
 class TestMethodsWithUnicode:
@@ -866,6 +1164,183 @@ class TestMethodsWithUnicode:
         res = np.array(res, dtype=dt)
         assert_array_equal(np.strings.expandtabs(buf), res)
 
+    @pytest.mark.parametrize("buf,width,fillchar,res", [
+        ('x', 2, '\U0001044E', 'x\U0001044E'),
+        ('x', 3, '\U0001044E', '\U0001044Ex\U0001044E'),
+        ('x', 4, '\U0001044E', '\U0001044Ex\U0001044E\U0001044E'),
+    ])
+    def test_center(self, buf, width, fillchar, res, dt):
+        buf = np.array(buf, dtype=dt)
+        fillchar = np.array(fillchar, dtype=dt)
+        res = np.array(res, dtype=dt)
+        assert_array_equal(np.strings.center(buf, width, fillchar), res)
+
+    @pytest.mark.parametrize("buf,width,fillchar,res", [
+        ('x', 2, '\U0001044E', 'x\U0001044E'),
+        ('x', 3, '\U0001044E', 'x\U0001044E\U0001044E'),
+        ('x', 4, '\U0001044E', 'x\U0001044E\U0001044E\U0001044E'),
+    ])
+    def test_ljust(self, buf, width, fillchar, res, dt):
+        buf = np.array(buf, dtype=dt)
+        fillchar = np.array(fillchar, dtype=dt)
+        res = np.array(res, dtype=dt)
+        assert_array_equal(np.strings.ljust(buf, width, fillchar), res)
+
+    @pytest.mark.parametrize("buf,width,fillchar,res", [
+        ('x', 2, '\U0001044E', '\U0001044Ex'),
+        ('x', 3, '\U0001044E', '\U0001044E\U0001044Ex'),
+        ('x', 4, '\U0001044E', '\U0001044E\U0001044E\U0001044Ex'),
+    ])
+    def test_rjust(self, buf, width, fillchar, res, dt):
+        buf = np.array(buf, dtype=dt)
+        fillchar = np.array(fillchar, dtype=dt)
+        res = np.array(res, dtype=dt)
+        assert_array_equal(np.strings.rjust(buf, width, fillchar), res)
+
+    @pytest.mark.parametrize("buf,sep,res1,res2,res3", [
+        ("āāāāĀĀĀĀ", "Ă", "āāāāĀĀĀĀ", "", ""),
+        ("āāāāĂĀĀĀĀ", "Ă", "āāāā", "Ă", "ĀĀĀĀ"),
+        ("āāāāĂĂĀĀĀĀ", "ĂĂ", "āāāā", "ĂĂ", "ĀĀĀĀ"),
+        ("𐌁𐌁𐌁𐌁𐌀𐌀𐌀𐌀", "𐌂", "𐌁𐌁𐌁𐌁𐌀𐌀𐌀𐌀", "", ""),
+        ("𐌁𐌁𐌁𐌁𐌂𐌀𐌀𐌀𐌀", "𐌂", "𐌁𐌁𐌁𐌁", "𐌂", "𐌀𐌀𐌀𐌀"),
+        ("𐌁𐌁𐌁𐌁𐌂𐌂𐌀𐌀𐌀𐌀", "𐌂𐌂", "𐌁𐌁𐌁𐌁", "𐌂𐌂", "𐌀𐌀𐌀𐌀"),
+        ("𐌁𐌁𐌁𐌁𐌂𐌂𐌂𐌂𐌀𐌀𐌀𐌀", "𐌂𐌂𐌂𐌂", "𐌁𐌁𐌁𐌁", "𐌂𐌂𐌂𐌂", "𐌀𐌀𐌀𐌀"),
+    ])
+    def test_partition(self, buf, sep, res1, res2, res3, dt):
+        buf = np.array(buf, dtype=dt)
+        sep = np.array(sep, dtype=dt)
+        res1 = np.array(res1, dtype=dt)
+        res2 = np.array(res2, dtype=dt)
+        res3 = np.array(res3, dtype=dt)
+        act1, act2, act3 = np.strings.partition(buf, sep)
+        assert_array_equal(act1, res1)
+        assert_array_equal(act2, res2)
+        assert_array_equal(act3, res3)
+        assert_array_equal(act1 + act2 + act3, buf)
+
+    @pytest.mark.parametrize("buf,sep,res1,res2,res3", [
+        ("āāāāĀĀĀĀ", "Ă", "", "", "āāāāĀĀĀĀ"),
+        ("āāāāĂĀĀĀĀ", "Ă", "āāāā", "Ă", "ĀĀĀĀ"),
+        ("āāāāĂĂĀĀĀĀ", "ĂĂ", "āāāā", "ĂĂ", "ĀĀĀĀ"),
+        ("𐌁𐌁𐌁𐌁𐌀𐌀𐌀𐌀", "𐌂", "", "", "𐌁𐌁𐌁𐌁𐌀𐌀𐌀𐌀"),
+        ("𐌁𐌁𐌁𐌁𐌂𐌀𐌀𐌀𐌀", "𐌂", "𐌁𐌁𐌁𐌁", "𐌂", "𐌀𐌀𐌀𐌀"),
+        ("𐌁𐌁𐌁𐌁𐌂𐌂𐌀𐌀𐌀𐌀", "𐌂𐌂", "𐌁𐌁𐌁𐌁", "𐌂𐌂", "𐌀𐌀𐌀𐌀"),
+    ])
+    def test_rpartition(self, buf, sep, res1, res2, res3, dt):
+        buf = np.array(buf, dtype=dt)
+        sep = np.array(sep, dtype=dt)
+        res1 = np.array(res1, dtype=dt)
+        res2 = np.array(res2, dtype=dt)
+        res3 = np.array(res3, dtype=dt)
+        act1, act2, act3 = np.strings.rpartition(buf, sep)
+        assert_array_equal(act1, res1)
+        assert_array_equal(act2, res2)
+        assert_array_equal(act3, res3)
+        assert_array_equal(act1 + act2 + act3, buf)
+
+    @pytest.mark.parametrize("method", ["strip", "lstrip", "rstrip"])
+    @pytest.mark.parametrize(
+        "source,strip",
+        [
+            ("λμ", "μ"),
+            ("λμ", "λ"),
+            ("λ" * 5 + "μ" * 2, "μ"),
+            ("λ" * 5 + "μ" * 2, "λ"),
+            ("λ" * 5 + "A" + "μ" * 2, "μλ"),
+            ("λμ" * 5, "μ"),
+            ("λμ" * 5, "λ"),
+    ])
+    def test_strip_functions_unicode(self, source, strip, method, dt):
+        src_array = np.array([source], dtype=dt)
+
+        npy_func = getattr(np.strings, method)
+        py_func = getattr(str, method)
+
+        expected = np.array([py_func(source, strip)], dtype=dt)
+        actual = npy_func(src_array, strip)
+
+        assert_array_equal(actual, expected)
+
+    @pytest.mark.parametrize("args", [
+        (None,),
+        (0,),
+        (1,),
+        (5,),
+        (15,),
+        (22,),
+        (-1,),
+        (-3,),
+        ([3, 4],),
+        ([-5, 5],),
+        ([0, -8],),
+        (1, 12),
+        (-12, 15),
+        (None, -1),
+        (0, [17, 6]),
+        ([1, 2], [-1, -2]),
+        (1, 11, 2),
+        (None, None, -1),
+        ([0, 10], [-1, 0], [2, -1]),
+    ])
+    def test_slice(self, args, dt):
+        buf = np.array(["Приве́т नमस्ते שָׁלוֹם", "😀😃😄😁😆😅🤣😂🙂🙃"],
+                       dtype=dt)
+        act = np.strings.slice(buf, *args)
+        bcast_args = tuple(np.broadcast_to(arg, buf.shape) for arg in args)
+        res = np.array([s[slice(*arg)]
+                        for s, arg in zip(buf, zip(*bcast_args))],
+                       dtype=dt)
+        assert_array_equal(act, res)
+
+
+class TestMixedTypeMethods:
+    def test_center(self):
+        buf = np.array("😊", dtype="U")
+        fill = np.array("*", dtype="S")
+        res = np.array("*😊*", dtype="U")
+        assert_array_equal(np.strings.center(buf, 3, fill), res)
+
+        buf = np.array("s", dtype="S")
+        fill = np.array("*", dtype="U")
+        res = np.array("*s*", dtype="S")
+        assert_array_equal(np.strings.center(buf, 3, fill), res)
+
+        with pytest.raises(ValueError, match="'ascii' codec can't encode"):
+            buf = np.array("s", dtype="S")
+            fill = np.array("😊", dtype="U")
+            np.strings.center(buf, 3, fill)
+
+    def test_ljust(self):
+        buf = np.array("😊", dtype="U")
+        fill = np.array("*", dtype="S")
+        res = np.array("😊**", dtype="U")
+        assert_array_equal(np.strings.ljust(buf, 3, fill), res)
+
+        buf = np.array("s", dtype="S")
+        fill = np.array("*", dtype="U")
+        res = np.array("s**", dtype="S")
+        assert_array_equal(np.strings.ljust(buf, 3, fill), res)
+
+        with pytest.raises(ValueError, match="'ascii' codec can't encode"):
+            buf = np.array("s", dtype="S")
+            fill = np.array("😊", dtype="U")
+            np.strings.ljust(buf, 3, fill)
+
+    def test_rjust(self):
+        buf = np.array("😊", dtype="U")
+        fill = np.array("*", dtype="S")
+        res = np.array("**😊", dtype="U")
+        assert_array_equal(np.strings.rjust(buf, 3, fill), res)
+
+        buf = np.array("s", dtype="S")
+        fill = np.array("*", dtype="U")
+        res = np.array("**s", dtype="S")
+        assert_array_equal(np.strings.rjust(buf, 3, fill), res)
+
+        with pytest.raises(ValueError, match="'ascii' codec can't encode"):
+            buf = np.array("s", dtype="S")
+            fill = np.array("😊", dtype="U")
+            np.strings.rjust(buf, 3, fill)
 
 
 class TestUnicodeOnlyMethodsRaiseWithBytes:
@@ -894,21 +1369,21 @@ class TestReplaceOnArrays:
     def test_replace_count_and_size(self, dt):
         a = np.array(["0123456789" * i for i in range(4)], dtype=dt)
         r1 = np.strings.replace(a, "5", "ABCDE")
-        assert r1.dtype.itemsize == check_itemsize(3*10 + 3*4, dt)
+        assert r1.dtype.itemsize == check_itemsize(3 * 10 + 3 * 4, dt)
         r1_res = np.array(["01234ABCDE6789" * i for i in range(4)], dtype=dt)
         assert_array_equal(r1, r1_res)
         r2 = np.strings.replace(a, "5", "ABCDE", 1)
-        assert r2.dtype.itemsize == check_itemsize(3*10 + 4, dt)
+        assert r2.dtype.itemsize == check_itemsize(3 * 10 + 4, dt)
         r3 = np.strings.replace(a, "5", "ABCDE", 0)
         assert r3.dtype.itemsize == a.dtype.itemsize
         assert_array_equal(r3, a)
         # Negative values mean to replace all.
         r4 = np.strings.replace(a, "5", "ABCDE", -1)
-        assert r4.dtype.itemsize == check_itemsize(3*10 + 3*4, dt)
+        assert r4.dtype.itemsize == check_itemsize(3 * 10 + 3 * 4, dt)
         assert_array_equal(r4, r1)
         # We can do count on an element-by-element basis.
         r5 = np.strings.replace(a, "5", "ABCDE", [-1, -1, -1, 1])
-        assert r5.dtype.itemsize == check_itemsize(3*10 + 4, dt)
+        assert r5.dtype.itemsize == check_itemsize(3 * 10 + 4, dt)
         assert_array_equal(r5, np.array(
             ["01234ABCDE6789" * i for i in range(3)]
             + ["01234ABCDE6789" + "0123456789" * 2], dtype=dt))
@@ -924,3 +1399,71 @@ class TestReplaceOnArrays:
                                         dtype=dt))
         r3 = np.strings.replace(a, ["0", "0,0", "0,0,0"], "X")
         assert_array_equal(r3, np.array(["X,X,X", "X,0", "X"], dtype=dt))
+
+
+class TestOverride:
+    @classmethod
+    def setup_class(cls):
+        class Override:
+
+            def __array_function__(self, *args, **kwargs):
+                return "function"
+
+            def __array_ufunc__(self, *args, **kwargs):
+                return "ufunc"
+
+        cls.override = Override()
+
+    @pytest.mark.parametrize("func, kwargs", [
+        (np.strings.center, dict(width=10)),
+        (np.strings.capitalize, {}),
+        (np.strings.decode, {}),
+        (np.strings.encode, {}),
+        (np.strings.expandtabs, {}),
+        (np.strings.ljust, dict(width=10)),
+        (np.strings.lower, {}),
+        (np.strings.mod, dict(values=2)),
+        (np.strings.multiply, dict(i=2)),
+        (np.strings.partition, dict(sep="foo")),
+        (np.strings.rjust, dict(width=10)),
+        (np.strings.rpartition, dict(sep="foo")),
+        (np.strings.swapcase, {}),
+        (np.strings.title, {}),
+        (np.strings.translate, dict(table=None)),
+        (np.strings.upper, {}),
+        (np.strings.zfill, dict(width=10)),
+    ])
+    def test_override_function(self, func, kwargs):
+        assert func(self.override, **kwargs) == "function"
+
+    @pytest.mark.parametrize("func, args, kwargs", [
+        (np.strings.add, (None, ), {}),
+        (np.strings.lstrip, (), {}),
+        (np.strings.rstrip, (), {}),
+        (np.strings.strip, (), {}),
+        (np.strings.equal, (None, ), {}),
+        (np.strings.not_equal, (None, ), {}),
+        (np.strings.greater_equal, (None, ), {}),
+        (np.strings.less_equal, (None, ), {}),
+        (np.strings.greater, (None, ), {}),
+        (np.strings.less, (None, ), {}),
+        (np.strings.count, ("foo", ), {}),
+        (np.strings.endswith, ("foo", ), {}),
+        (np.strings.find, ("foo", ), {}),
+        (np.strings.index, ("foo", ), {}),
+        (np.strings.isalnum, (), {}),
+        (np.strings.isalpha, (), {}),
+        (np.strings.isdecimal, (), {}),
+        (np.strings.isdigit, (), {}),
+        (np.strings.islower, (), {}),
+        (np.strings.isnumeric, (), {}),
+        (np.strings.isspace, (), {}),
+        (np.strings.istitle, (), {}),
+        (np.strings.isupper, (), {}),
+        (np.strings.rfind, ("foo", ), {}),
+        (np.strings.rindex, ("foo", ), {}),
+        (np.strings.startswith, ("foo", ), {}),
+        (np.strings.str_len, (), {}),
+    ])
+    def test_override_ufunc(self, func, args, kwargs):
+        assert func(self.override, *args, **kwargs) == "ufunc"
